<?php

namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Util\Utilities;
use Teknisa\SM\Service\Sm00005000ParametrizacoesServicesService;

class FctOrcamentoGridController extends SMCrud {
    
    protected $dataSourceName = "fct_orcamento_grid";
    protected $parametrogeral;
    
    public function __construct($crudService, $environment, Sm00005000ParametrizacoesServicesService $parametrogeral) {
        parent::__construct($crudService, $environment);
        $this->parametrogeral = $parametrogeral;
    }

    /**
     * @param Request\Row $request
     */
    public function validarperfilacesso (Request $request, Response $response){
        try {
            $result = false;
            $row = $request->getRow();
            $nrparcnegocio = $row[0];
            $nrorg         = $row[1];
            $derivadode    = $row[2];
            $perfilacessousuario = $this->parametrogeral->validarperfilacesso($nrparcnegocio);
            if($derivadode == 'P') {
                $perfilacessoparametrizado = $this->parametrogeral->buscarperfilacesso($nrorg, 'P');
            } else if ($derivadode == 'C'){
                $perfilacessoparametrizado = $this->parametrogeral->buscarperfilacesso($nrorg, 'C');
            }
            
            if($perfilacessoparametrizado){
                $perfilacessoparametrizadoarray = array();
                $perfilacessousuario[0]["NRPERFILACESSO"];
                foreach($perfilacessoparametrizado as $perfil){
                    array_push($perfilacessoparametrizadoarray,$perfil["NRPERFILACESSO"]);
                }

                if (count($perfilacessousuario) > 0){
                    if(!in_array($perfilacessousuario[0]["NRPERFILACESSO"], $perfilacessoparametrizadoarray))
                        $result = true;
                }
            }

            $response->setDataSets(array(new DataSet($this->dataSourceName, [$result])));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Faz a busca dos registros no banco de dados, trazendo os Escopos de Proposta concatenados em uma coluna só.
     * 
     * @param Request\Filter $request
     * @param Response $response
     */
    public function find(Request\Filter $request, Response $response) {
        try {
            $idUtilizaOper = $this->parametrogeral->getIdUtilizaHierarquiaOper($this->environment->getNrorg());
            //$request->getFilterCriteria()->addCondition("IDPROPALIMENTICIA", "=", "N");
            if($idUtilizaOper == 'N')
                $this->dataSourceName = 'fct_orcamento_semhieraroper';
            else
                $request->getFilterCriteria()->addCondition("CDOPERADOR", "=", $this->environment->getCdOperador());
            
            parent::find($request, $response);
            $rows = $response->getDataSets()[0]->getRows();
            foreach ($rows as &$row) {
                $row["NRTIPOORCAMENTO"]   = Utilities::getArrayDeString($row["NRTIPOORCAMENTO"], true);
                $row["IDUTILIZAVARPRECO"] = Utilities::getArrayDeString($row["IDUTILIZAVARPRECO"]);
            }
            unset($row);
            $this->dataSourceName = "fct_orcamento_grid";
            $response->setDataSets(array(new DataSet($this->dataSourceName, $rows)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
}

?>