<?php

namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00001100LancamentoPropostaFactory {
    protected $connection;
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    /**
     * busca o caminho do modelo do documento anexado na proposta
     */
    public function buscaDocAnexado(array $params) {
       $sql = "SELECT NRSEQORCADOC,
                      NMARQUIVO,
                      DSCAMINHOARQ,
                      DSTIPO,
                      VRTAMANHO 
                 FROM FCT_ORCAMENTODOC 
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll(); 
    }
    /**
     * Atualiza a Proposta ao aceitá-la.
     * 
     * @param mixed[] $params
     */
    public function atualizarSituacaoProposta(array $params) {
        $sql = "UPDATE FCT_ORCAMENTO
                   SET IDSITUACAOORCAMENTO = :IDSITUACAOORCAMENTO,
                       DTACEITEORCA        = :DTACEITEORCA,
                       CDOPERULTATU        = :CDOPERULTATU,
                       DTULTATU            = SYSDATE
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Atualiza a tabela de documento da proposta quando a situação da mesma 
     * muda 
    **/
    public function atualizaDocumentoSituacao(array $params){
        $sql = "UPDATE FCT_ORCAMENTODOC
                SET NRSEQMODDOCFCT 	    = :NRSEQMODDOCFCT,
                	NMARQUIVO      	    = :NMARQUIVO,
                	DSCAMINHOARQ   	    = :DSCAMINHOARQ,
                	DSTIPO         	    = :DSTIPO,
                	VRTAMANHO           = :VRTAMANHO,
                	IDSITUACAOORCAMENTO = :IDSITUACAOORCAMENTO
                WHERE NRSEQORCAMENTO    = :NRSEQORCAMENTO
                  AND NRTIPOORCAMENTO   = :NRTIPOORCAMENTO
                  AND NRORG             = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Retorna se o parceiro deve ser convertido ou não
    **/
    public function getIdConverteProspectEmCliente(array $params) {
        $sql = "SELECT IDCONVCLIENTE
                  FROM FCT_PARAMETROGERAL
                 WHERE NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Retorna se há um cliente cadastrado com o número do parceiro
    **/
    public function getCountClienteProspect(array $params) {
        $sql = "SELECT COUNT(*) AS QTDE
                  FROM CLIENTE
                 WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                   AND NRORG         = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Seleciona o documento do contrato gerado
    **/
    public function getDocumentosContratosGerados(array $params) {
        $sql = "SELECT DOC.NRSEQCONTRATO,
                       DOC.NRSEQCONTDOC,
                       DOC.NMARQUIVO,
                       DOC.DSCAMINHOARQ,
                       DOC.DSTIPO,
                       DOC.VRTAMANHO
                  FROM FCT_CONTRATODOC DOC
                 INNER JOIN FCT_REFEORCACONT REFE
                    ON DOC.NRSEQCONTRATO = REFE.NRSEQCONTRATO
                   AND DOC.NRORG         = REFE.NRORG
                 WHERE REFE.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND REFE.NRORG          = :NRORG
                   AND REFE.IDATIVO        = 'S'";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Retorna o numero dos contratos e  
     **/
    public function getItemContratos(array $params){
        $sql = "SELECT ITCON.NRITEMOCONTRATO, 
                    ITCON.NRSEQCONTRATO
                FROM FCT_ITEMCONTRATO ITCON
                INNER JOIN FCT_REFEORCACONT REF
                    ON REF.NRSEQCONTRATO = ITCON.NRSEQCONTRATO
                    AND REF.NRORG        = ITCON.NRORG
                WHERE REF.NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND ITCON.CDSERVORCA = :CDSERVORCA
                    AND ITCON.NRORG      = :NRORG
                    AND REF.IDATIVO      = 'S'";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Retorna os dados do documento da proposta de acordo com a situação
    **/
    public function getDadosDocumentoSituacao(array $params){
        $sql = "SELECT ESCOPODOC.NRSEQMODDOCFCT,
                     DOC.NMARQUIVO,
                     DOC.DSCAMINHOARQ,
                     DOC.DSTIPO,
                     DOC.VRTAMANHO,
                     REL.NRTIPOORCAMENTO,
                     ESCOPODOC.IDSITUACAOORCAMENTO
                FROM FCT_ORCAESCOPODOC ESCOPODOC
                INNER JOIN FCT_MODELODOC DOC
                  ON ESCOPODOC.NRSEQMODDOCFCT  = DOC.NRSEQMODDOCFCT
                  AND ESCOPODOC.NRORG          = DOC.NRORG
                INNER JOIN FCT_RELACESCOPOMODELO REL
                  ON REL.NRRELACESCOPOMODELO = ESCOPODOC.NRRELACESCOPOMODELO
                  AND REL.NRORG              = ESCOPODOC.NRORG
                WHERE REL.NRORCAMENTOMODELO = (SELECT NRORCAMENTOMODELO FROM FCT_ORCAMENTO WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO)
                  AND REL.NRORG             = :NRORG
                  AND (IDSITUACAOORCAMENTO  = :IDSITUACAOORCAMENTO
                        OR (IDSITUACAOORCAMENTO IS NULL AND (SELECT COUNT(IDSITUACAOORCAMENTO)
                                                              FROM FCT_ORCAESCOPODOC
                                                              WHERE IDSITUACAOORCAMENTO   = :IDSITUACAOORCAMENTO
                                                                AND NRRELACESCOPOMODELO   = ESCOPODOC.NRRELACESCOPOMODELO) = 0))";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Altera o caminho do documento de um contrato gerado por uma proposta
    **/
    public function atualizarCaminhoDocumentoContrato(array $params) {
        $sql = "UPDATE FCT_CONTRATODOC
                   SET DSCAMINHOARQ = :DSCAMINHOARQ
                 WHERE NRSEQCONTDOC  = :NRSEQCONTDOC
                   AND NRSEQCONTRATO = :NRSEQCONTRATO
                   AND NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Atualiza o documento anexado na proposta
     */
    public function AnexaOrcamentoDoc(array $params) {
       $sql = "UPDATE FCT_ORCAMENTODOC
                  SET NMARQUIVO    = :NMARQUIVO,
                      DSCAMINHOARQ = :DSCAMINHOARQ,
                      VRTAMANHO    = :VRTAMANHO,
                      DSTIPO       = :DSTIPO,
                      DTULTATU     = SYSDATE,
                      CDOPERULTATU = :CDOPERULTATU,
                      NRORGULTATU  = :NRORGULTATU
                WHERE NRSEQORCADOC   = :NRSEQORCADOC
                  AND NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Retorna os Documentos da Proposta.
     * 
     * @param mixed[] $params
     * @return mixed[]
     */
    public function getDocumentosProposta(array $params) {
        $sql = "SELECT NRSEQORCADOC,
                       TXMODELOORCADOC,
                       NMARQUIVO,
                       DSCAMINHOARQ,
                       DSTIPO,
                       VRTAMANHO,
                       NRORG
                  FROM FCT_ORCAMENTODOC
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Exclui os escopos da proposta.
     **/
    public function excluirEscopos(array $params) {
        $sql = "DELETE FROM FCT_RELACESCOPOPROPOSTA
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * A função "buscaModeloDocumento" busca o modelo de documentos para cada 
     * escopo a fim de ser inserido na tabela FCT_RELACESCOPOPROPOSTA
     **/
    
    public function buscaModeloDocumento(array $params) {
        $sql = "SELECT ESCOPODOC.NRSEQMODDOCFCT, REL.CDFILIAL
                  FROM FCT_ORCAESCOPODOC ESCOPODOC
                      INNER JOIN FCT_RELACESCOPOMODELO REL
                          ON REL.NRRELACESCOPOMODELO = ESCOPODOC.NRRELACESCOPOMODELO
                          AND REL.NRORG = ESCOPODOC.NRORG
                 WHERE REL.NRORCAMENTOMODELO = :NRORCAMENTOMODELO
                   AND REL.NRTIPOORCAMENTO   = :NRTIPOORCAMENTO
                   AND REL.NRORG             = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * A função "buscaFilialHierarquia" busca a filial vinculado ao 
     * operador na hierarquiia de operadores (CRM_HIERARQUIAOPER)
     * a fim de ser inserido na tabela FCT_RELACESCOPOPROPOSTA
     **/
    public function buscaFilialHierarquia(array $params) {
        $sql = "SELECT CDFILIAL
                FROM CRM_HIERARQUIAOPER
                WHERE CDOPERADOR = :CDOPERADOR
                    AND NRORG    = :NRORG
                    AND ROWNUM   = 1  
                ORDER BY DTULTATU DESC";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Exclui a proposta.
     **/
    public function excluirProposta(array $params) {
        $sql = "DELETE FROM FCT_ORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Salva os serviços ligados a uma proposta
    **/
    public function inserirServicosProposta(array $params){
        $sql = "INSERT INTO FCT_RELACSERVORCAMENTO(NRRELACSERVORCA, NRSEQORCAMENTO, IDESCOLHIDO, 
                    IDTIPOELEMENTO,  CDSERVORCA,     NRCOMPSERVICO,   VRCUSTOSERV,   VRTOTMENSALSERV, 
                    QTSEGUNDA,       QTTERCA,        QTQUARTA,        QTQUINTA,      QTSEXTA,  
                    QTSABADO,        QTDOMINGO,      CDTPMEDIDASERV,  QTMEDSERVORCA, IDSERVOBRIGCOMP, 
                    IDTPDESCORCA,    VRPERCDESCORCA, VRTOTMSLSAJUSTE, IDELEMENTO,    IDELEMENTOPAI, 
                    NRTIPOORCAMENTO, NRORG,          DTINCLUSAO,      DTULTATU,      NRORGINCLUSAO, 
                    CDOPERINCLUSAO,  NRORGULTATU,    CDOPERULTATU,    IDATIVO,       QTMENSAL)
                VALUES(:NRRELACSERVORCA, :NRSEQORCAMENTO, :IDESCOLHIDO, 
                    :IDTIPOELEMENTO,  :CDSERVORCA,     :NRCOMPSERVICO,   :VRCUSTOSERV,   :VRTOTMENSALSERV, 
                    :QTSEGUNDA,       :QTTERCA,        :QTQUARTA,        :QTQUINTA,      :QTSEXTA,  
                    :QTSABADO,        :QTDOMINGO,      :CDTPMEDIDASERV,  :QTMEDSERVORCA, :IDSERVOBRIGCOMP, 
                    :IDTPDESCORCA,    :VRPERCDESCORCA, :VRTOTMSLSAJUSTE, :IDELEMENTO,    :IDELEMENTOPAI, 
                    :NRTIPOORCAMENTO, :NRORG,          SYSDATE,           SYSDATE,       :NRORGINCLUSAO, 
                    :CDOPERINCLUSAO,  :NRORGULTATU,    :CDOPERULTATU,    :IDATIVO,       :QTMENSAL)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Salva os serviços ligados a uma proposta
    **/
    public function inserirMaoObrasProposta(array $params){
        $sql = "INSERT INTO FCT_RELACFORMACAOPROP 
                (NRRELACFORMACAOPROP, NRFORMPRECOSERVICO,  NRSEQORCAMENTO,  DTULTATU,
                    NRITEMORCAMENTO,  NRORG,               IDATIVO,        DTINCLUSAO, 
                    NRORGINCLUSAO,    CDOPERINCLUSAO,      NRORGULTATU,    CDOPERULTATU,
                    QTFORMACAOPROP,   CDSERVORCARELAC)
                VALUES(:NRRELACFORMACAOPROP,           :NRFORMPRECOSERVICO,  :NRSEQORCAMENTO, SYSDATE, 
                    :NRITEMORCAMENTO, :NRORG,          :IDATIVO,             SYSDATE,          
                    :NRORGINCLUSAO,   :CDOPERINCLUSAO, :NRORGULTATU,         :CDOPERULTATU,
                    :QTFORMACAOPROP,  :CDSERVORCARELAC)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Salva as despesas ligados a uma proposta
    **/
    public function inserirDespesasProposta(array $params){
        $sql = "INSERT INTO FCT_RELACDESPPROPOSTA (NRDESPPROP, CDDESPESA, NRSEQORCAMENTO, DTULTATU,
                        VRDESPESAORCA,     NRORG,          IDATIVO,     DTINCLUSAO, 
                        NRORGINCLUSAO,     CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU,
                        CDPERIODODESPPROP, VRUNITDESPESA)
                VALUES(:NRDESPPROP,         :CDDESPESA,      :NRSEQORCAMENTO, SYSDATE, 
                        :VRDESPESAORCA,     :NRORG,          :IDATIVO,        SYSDATE,          
                        :NRORGINCLUSAO,     :CDOPERINCLUSAO, :NRORGULTATU,   :CDOPERULTATU,
                        :CDPERIODODESPPROP, :VRUNITDESPESA)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);                
    }
    /**
     * Salva as taxas ligados a uma proposta
    **/
    public function inserirTaxasProposta(array $params){
        $sql = "INSERT INTO FCT_RELACTAXAPROPOSTA (NRTAXAPROP, CDTAXA, NRSEQORCAMENTO, DTULTATU,
                        VRPERCTAXAORC,   NRORG,          IDATIVO,     DTINCLUSAO, 
                        NRORGINCLUSAO,   CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU)
                VALUES(:NRTAXAPROP,     :CDTAXA,         :NRSEQORCAMENTO, SYSDATE, 
                        :VRPERCTAXAORC, :NRORG,          :IDATIVO,        SYSDATE,          
                        :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU,   :CDOPERULTATU)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);                
    }
    /**
     * Altera na tabela de itens da formação de preco
     * o valor do orcamento dos itens utilizados
    **/
    public function updateItensFormacaoProposta(array $params, $formacoes){
        $sql = "UPDATE FCT_CALCFORMPRECSERV_ITPRO
                SET NRSEQORCAMENTO      = :NRSEQORCAMENTO,
                    NRRELACFORMACAOPROP = :NRRELACFORMACAOPROP,
                    NRITEMORCAMENTO     = 0
                WHERE NRSEQORCAMENTO        = 0
                    AND NRITEMORCAMENTO     = -1
                    AND NRORG               = :NRORG
                    AND NRFORMPRECOSERVICO  = :NRFORMPRECOSERVICO
                    AND NRRELACFORMACAOPROP = :IDFORMACAO";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);            
    }
    /**
     * Salva os investimentos ligados a uma proposta
    **/
    public function inserirInvestimentosProposta(array $params){
        $sql = "INSERT INTO FCT_RELACINVESTPROPOSTA (NRINVESTPROP, NRINVESTIMENTO, NRSEQORCAMENTO, DTULTATU,
                        VRUNITINVESTORCA,   NRORG,          IDATIVO,          DTINCLUSAO, 
                        NRORGINCLUSAO,      CDOPERINCLUSAO, NRORGULTATU,      CDOPERULTATU, 
                        VRTOTINVETORCA,     QTINVESTORCA,   QTMESARMOTIZACAO, VRPARCELA)
                VALUES(:NRINVESTPROP,      :NRINVESTIMENTO, :NRSEQORCAMENTO,    SYSDATE, 
                        :VRUNITINVESTORCA, :NRORG,          :IDATIVO,           SYSDATE,          
                        :NRORGINCLUSAO,    :CDOPERINCLUSAO, :NRORGULTATU,       :CDOPERULTATU, 
                        :VRTOTINVETORCA,   :QTINVESTORCA,   :QTMESARMOTIZACAO,  :VRPARCELA)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);                
    }
    /**
     * Retorna as informações do serviço para a widget de custo do serviço
     * com o código da composição que eles pertencem
    **/
    public function getElementosComposicaoServico($nrtipoorcamento, $nrorg, $nrcomposicao){
        $sql = "SELECT DISTINCT IDELEMENTO,
                    IDELEMENTOPAI,
                    IDTIPOELEMENTO,
                    CDTPMEDIDAVARIAVEL,
                    CDELEMENTO,
                    CDSERVORCARELAC,
                    NMSERVORCARELAC,
                    NRTIPOORCAMENTO,
                    NMTIPOORCAMENTO,
                    IDALTQTDMEDIDAPROP,
                    IDFATURACONTR,
                    CDMODPGTO,
                    NRGRUPOFATURA,
                    QTPARCELASFATU,
                    NRDIAVENCPARCELASFATU,
                    IDUTILIZAVARPRECO,
                    CDTPMEDIDASERV,
                    QTMEDIDASERV,
                    VRUNITARIO,
                    VRPRECO,
                    DTFIMVALPBSERV,
                    NRORG,
                    NULL AS QTMEDIDAVARIAVEL,
                    IDELEMENTORAIZ,
                    NMELEMENTO,
                    0 AS VRCUSTOSERV,
                    NRCOMPSERVICO,
                    '00:00' AS HRENTRADA,
                    '00:00' AS HRSAIDA,
                    0 AS QTDOMINGO,
                    0 AS QTSABADO,
                    0 AS QTMENSAL,
                    0 AS QTSEGUNDA,
                    0 AS QTTERCA,
                    0 AS QTQUARTA,
                    0 AS QTQUINTA,
                    0 AS QTSEXTA,
                    0 AS QTDIAUTIL,
                    0 AS QTFIMSEMANA
                    FROM (SELECT DISTINCT COMPOSICAO.IDELEMENTO,
                         COMPOSICAO.IDELEMENTOPAI,
                         COMPOSICAO.IDTIPOELEMENTO,
                         COMPOSICAO.CDTPMEDIDAVARIAVEL,
                         COMPOSICAO.CDELEMENTO,
                         COMPOSICAO.CDSERVORCARELAC,
                         COMPOSICAO.NMSERVORCARELAC,
                         COMPOSICAO.NRTIPOORCAMENTO,
                         COMPOSICAO.NMTIPOORCAMENTO,
                         COMPOSICAO.IDALTQTDMEDIDAPROP,
                         COMPOSICAO.IDFATURACONTR,
                         COMPOSICAO.CDMODPGTO,
                         COMPOSICAO.NRGRUPOFATURA,
                         COMPOSICAO.QTPARCELASFATU,
                         COMPOSICAO.NRDIAVENCPARCELASFATU,
                         COMPOSICAO.IDUTILIZAVARPRECO,
                         COMPOSICAO.CDTPMEDIDASERV,
                         COMPOSICAO.QTMEDIDASERV,
                         COMPOSICAO.VRPRECO AS VRUNITARIO,
                         COMPOSICAO.VRPRECO * COMPOSICAO.QTMEDIDASERV AS VRPRECO,
                         TO_CHAR(COMPOSICAO.DTFIMVALPBSERV, 'DD/MM/YYYY') AS DTFIMVALPBSERV,
                         COMPOSICAO.NRORG,
                         COMPOSICAO.IDELEMENTORAIZ,
                         COMPOSICAO.NRCOMPSERVICO,
                         ELE.NMSERVORCA AS NMELEMENTO
                        FROM (SELECT DISTINCT ITEM.NRCOMPSERVICO,
                             ITEM.IDELEMENTO,
                             ITEM.IDELEMENTOPAI,
                             ELEMENTO.IDTIPOELEMENTO,
                             ELEMENTO.CDTPMEDIDAVARIAVEL,
                             ITEM.CDSERVORCA AS CDELEMENTO,
                             ITEM.IDSERVOBRIGCOMP,
                             SERVRELAC.CDSERVORCARELAC,
                             SERVICO.NMSERVORCA AS NMSERVORCARELAC,
                             ITESCOPO.NRTIPOORCAMENTO,
                             ESCOPO.NMTIPOORCAMENTO,
                             ESCOPO.IDALTQTDMEDIDAPROP,
                             ESCOPO.IDUTILIZAVARPRECO,
                             DECODE(PARAMSRV.NRSEQFATUSRVESCO, NULL, PARAMESC.IDFATURACONTR, PARAMSRV.IDFATURACONTR) AS IDFATURACONTR,
                             DECODE(PARAMSRV.NRSEQFATUSRVESCO, NULL, PARAMESC.CDMODPGTO, PARAMSRV.CDMODPGTO) AS CDMODPGTO,
                             DECODE(PARAMSRV.NRSEQFATUSRVESCO, NULL, PARAMESC.NRGRUPOFATURA, PARAMSRV.NRGRUPOFATURA) AS NRGRUPOFATURA,
                             DECODE(PARAMSRV.NRSEQFATUSRVESCO, NULL, PARAMESC.QTPARCELASFATU, PARAMSRV.QTPARCELASFATU) AS QTPARCELASFATU,
                             DECODE(PARAMSRV.NRSEQFATUSRVESCO, NULL, PARAMESC.NRDIAVENCPARCELASFATU, PARAMSRV.NRDIAVENCPARCELASFATU) AS NRDIAVENCPARCELASFATU,
                             ITPRECO.CDTPMEDIDASERV,
                             DECODE(ITPRECO.VRPRECO, NULL, NULL, SERVRELAC.QTMEDIDASERV) AS QTMEDIDASERV,
                             ITPRECO.VRPRECO,
                             ITPRECO.DTFIMVALPBSERV,
                             ITEM.NRORG,
                             (SELECT DISTINCT IDELEMENTO
                                   FROM FCT_ITCOMPSERVICO
                                  WHERE NRCOMPSERVICO = ITEM.NRCOMPSERVICO
                                    AND CONNECT_BY_ISLEAF = 1
                                CONNECT BY PRIOR IDELEMENTOPAI = IDELEMENTO
                                    AND PRIOR NRCOMPSERVICO = ITEM.NRCOMPSERVICO
                                  START WITH IDELEMENTO = ITEM.IDELEMENTO) AS IDELEMENTORAIZ
                        FROM FCT_ITCOMPSERVICO ITEM
                        LEFT JOIN FCT_RELACSERVICO SERVRELAC
                            ON ITEM.CDSERVORCA = SERVRELAC.CDSERVORCA
                            AND ITEM.NRORG      = SERVRELAC.NRORG
                        LEFT JOIN FCT_SERVICOORCA SERVICO
                            ON SERVRELAC.CDSERVORCARELAC = SERVICO.CDSERVORCA
                            AND SERVRELAC.NRORG           = SERVICO.NRORG
                        LEFT JOIN FCT_SERVICOORCA ELEMENTO
                            ON ITEM.CDSERVORCA      = ELEMENTO.CDSERVORCA
                            AND ITEM.NRORG           = ELEMENTO.NRORG 
                        LEFT JOIN FCT_ITESCOPOPROPOSTA ITESCOPO
                            ON SERVRELAC.CDSERVORCARELAC = ITESCOPO.CDSERVORCA
                            AND SERVRELAC.NRORG           = ITESCOPO.NRORG
                            AND ITESCOPO.NRTIPOORCAMENTO IN ($nrtipoorcamento)
                        LEFT JOIN FCT_TIPOORCAMENTO ESCOPO
                            ON ITESCOPO.NRTIPOORCAMENTO = ESCOPO.NRTIPOORCAMENTO
                            AND ITESCOPO.NRORG           = ESCOPO.NRORG
                        LEFT JOIN FCT_PARAMFATESCOPO PARAMESC
                            ON ESCOPO.NRTIPOORCAMENTO = PARAMESC.NRTIPOORCAMENTO
                            AND ESCOPO.NRORG           = PARAMESC.NRORG
                        LEFT JOIN FCT_PARAMFATSRVESCO PARAMSRV
                            ON ITESCOPO.NRITESCOPOPROPOSTA = PARAMSRV.NRITESCOPOPROPOSTA
                            AND ITESCOPO.NRORG              = PARAMSRV.NRORG
                        LEFT JOIN FCT_ITPRECBASESERV ITPRECO
                            ON ESCOPO.CDPRECBASESERV = ITPRECO.CDPRECBASESERV
                            AND ESCOPO.NRORG          = ITPRECO.NRORG
                            AND ITESCOPO.CDSERVORCA   = ITPRECO.CDSERVORCA
                            WHERE ITEM.NRCOMPSERVICO IN ($nrcomposicao)
                          AND ITEM.NRORG = :NRORG
                          AND TRUNC(NVL(ITPRECO.DTINIPBSERV,SYSDATE)) <= TRUNC(SYSDATE)) COMPOSICAO
                    LEFT JOIN FCT_SERVICOORCA ELE
                        ON COMPOSICAO.IDELEMENTO = ELE.IDELEMENTO
                        AND COMPOSICAO.NRORG     = ELE.NRORG)
                    order by IDELEMENTO";
        $std = $this->connection->prepare($sql);
        $params = array(
            "NRORG"             => $nrorg,
        );
        $std->execute($params);   
        return $std->fetchAll();
    }
    /**
     * Retorna as informações do serviço para a widget de custo do serviço
     * sem o código da composição que eles pertencem
    **/
    public function getElementosComposicaoServicoSemNrComposicao($nrtipoorcamento, $nrorg, $nrcomposicao) {
        $sql = "SELECT CDSERVORCARELAC,
                        NMSERVORCARELAC,
                        NRORG,
                        IDELEMENTORAIZ,
                        IDELEMENTOPAI,
                        IDELEMENTO, 
                        NMELEMENTO,
                        VRCUSTOSERV,
                        LISTAGG(NRCOMPSERVICO, '_') WITHIN GROUP(ORDER BY NRCOMPSERVICO) AS NRCOMPSERVICO
                FROM (SELECT DECODE(NVL(CDSERVORCARELAC, -1), -1, NULL, IDELEMENTORAIZ) AS IDELEMENTOPAI,
                        CASE 
                          WHEN CDSERVORCARELAC IS NULL THEN IDELEMENTORAIZ
                          ELSE IDELEMENTORAIZ || '_' || CDSERVORCARELAC
                        END AS IDELEMENTO,
                        CDSERVORCARELAC,
                        NMSERVORCARELAC,
                        IDELEMENTORAIZ,
                        NRCOMPSERVICO,
                        ELESERVORCA.NRORG,
                        0 AS VRCUSTOSERV,
                        SERVRAIZ.NMSERVORCA AS NMELEMENTO
                      FROM (SELECT DISTINCT SERVRELAC.CDSERVORCARELAC,
                               SERVICO.NMSERVORCA AS NMSERVORCARELAC,
                               ITEM.NRORG,
                               (SELECT DISTINCT IDELEMENTO
                                   FROM FCT_ITCOMPSERVICO
                                  WHERE NRCOMPSERVICO = ITEM.NRCOMPSERVICO
                                    AND CONNECT_BY_ISLEAF = 1
                                CONNECT BY PRIOR IDELEMENTOPAI = IDELEMENTO
                                    AND PRIOR NRCOMPSERVICO = ITEM.NRCOMPSERVICO
                                  START WITH IDELEMENTO = ITEM.IDELEMENTO) AS IDELEMENTORAIZ,
                            ITEM.NRCOMPSERVICO  
                            FROM FCT_ITCOMPSERVICO ITEM
                            LEFT JOIN FCT_RELACSERVICO SERVRELAC
                              ON ITEM.CDSERVORCA = SERVRELAC.CDSERVORCA
                              AND ITEM.NRORG      = SERVRELAC.NRORG
                            LEFT JOIN FCT_SERVICOORCA SERVICO
                              ON SERVRELAC.CDSERVORCARELAC = SERVICO.CDSERVORCA
                              AND SERVRELAC.NRORG           = SERVICO.NRORG
                            INNER JOIN FCT_ITESCOPOPROPOSTA ITESCOPO
                                ON SERVRELAC.CDSERVORCARELAC = ITESCOPO.CDSERVORCA
                                AND SERVRELAC.NRORG           = ITESCOPO.NRORG
                                AND ITESCOPO.NRTIPOORCAMENTO IN ($nrtipoorcamento)
                            WHERE ITEM.NRORG = :NRORG
                              AND NRCOMPSERVICO IN ($nrcomposicao))ELESERVORCA
                      LEFT JOIN FCT_SERVICOORCA SERVRAIZ
                        ON ELESERVORCA.IDELEMENTORAIZ = SERVRAIZ.IDELEMENTO
                        AND ELESERVORCA.NRORG         = SERVRAIZ.NRORG) ELEMENTO
                GROUP BY CDSERVORCARELAC,
                         NMSERVORCARELAC,
                         NRORG,
                         IDELEMENTORAIZ,
                         IDELEMENTOPAI,
                         IDELEMENTO,
                         NMELEMENTO,
                         VRCUSTOSERV
                ORDER BY IDELEMENTORAIZ";
        $std = $this->connection->prepare($sql);
        $params = array(
            "NRORG"             => $nrorg,
        );
        $std->execute($params);   
        return $std->fetchAll();
    }
    /**
     * Insere os Itens da Proposta.
     * @param string[] $params
     */
    public function inserirItensProposta(array $params) {
        $sql = "INSERT INTO FCT_ITEMORCAMENTO (
                    NRITEMORCAMENTO,        NRSEQORCAMENTO,         CDSERVORCA,         NMITEMORCAMENTO,
                    DSQTITEMORCA,           QTITEMORCAMENTO,        CDTPMEDIDASERV,     QTMEDIDAORCA,
                    VRUNITITEMORCA,         VRTOTALITEMORCA,        IDITEMOBRIGORCA,    IDSOMASUBITENS,
                    IDSOMANOITEMPAI,        IDSOMANOTOTALORCA,      IDIMPRESSAOORCA,    IDPRECOLOCACAO,
                    IDMULTMEDVRTOT,         IDBLOQALTORC,           IDADICITEMORC,      IDADICCOBRITEMORC,
                    IDTPDESCORCA,           IDTRANSPORTADORA,       IDATIVO,            NRORG,
                    CDOPERINCLUSAO,         NRORGINCLUSAO,          DTINCLUSAO,         NRTIPOORCAMENTO,
                    IDFATURAORCA,           CDMODPGTO,              NRGRUPOFATURA,      QTPARCELASFATU,
                    NRDIAVENCPARCELASFATU,  VRPERCDESCORCA,         VRITEMSEMDESC,      NMREDUITEMORCA,
                    IDELEMENTO,            CDTPMEDIDAVARIAVEL,    NRITEMORCAMENTOPAI
                )
                VALUES (
                    :NRITEMORCAMENTO,       :NRSEQORCAMENTO,        :CDSERVORCA,        :NMITEMORCAMENTO,
                    :DSQTITEMORCA,          :QTITEMORCAMENTO,       :CDTPMEDIDASERV,    :QTMEDIDAORCA,
                    :VRUNITITEMORCA,        :VRTOTALITEMORCA,       :IDITEMOBRIGORCA,   :IDSOMASUBITENS,
                    :IDSOMANOITEMPAI,       :IDSOMANOTOTALORCA,     :IDIMPRESSAOORCA,   :IDPRECOLOCACAO,
                    :IDMULTMEDVRTOT,        :IDBLOQALTORC,          :IDADICITEMORC,     :IDADICCOBRITEMORC,
                    :IDTPDESCORCA,          :IDTRANSPORTADORA,      :IDATIVO,           :NRORG,
                    :CDOPERINCLUSAO,        :NRORGINCLUSAO,         SYSDATE,            :NRTIPOORCAMENTO,
                    :IDFATURAORCA,          :CDMODPGTO,             :NRGRUPOFATURA,     :QTPARCELASFATU,
                    :NRDIAVENCPARCELASFATU, :VRPERCDESCORCA,        :VRITEMSEMDESC,     :NMREDUITEMORCA,
                    :IDELEMENTO,            :CDTPMEDIDAVARIAVEL,    :NRITEMORCAMENTOPAI
                )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Insere as informações finais dos valores de cada serviço final
     * da proposta - tela de resultado -> aba serviços
    **/
    public function inserirResultadoOrcamento(array $params){
        $sql = "INSERT INTO FCT_RESULTORCALIMENT(NRRESULTORCA, NRSEQORCAMENTO, VRCUSTOINFORMADO, VRCUSTOIDEAL, IDELEMENTORAIZ, 
                		NRCOMPSERVICORAIZ, QTTOTALSERV, VRTOTSERV, IDATIVO, NRORG, 
                		CDOPERINCLUSAO, NRORGINCLUSAO, DTINCLUSAO, VRPERCSERVICO)
                VALUES (:NRRESULTORCA, :NRSEQORCAMENTO, :VRCUSTOINFORMADO, :VRCUSTOIDEAL, :IDELEMENTORAIZ,
                		:NRCOMPSERVICORAIZ, :QTTOTALSERV, :VRTOTSERV, :IDATIVO, :NRORG, 
                		:CDOPERINCLUSAO, :NRORGINCLUSAO, SYSDATE, :VRPERCSERVICO)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /*public function getServicosPropostaInserindo($nrtipoorcamento, $nrorg, $nrmodelo){
        $sql = "SELECT TO_CHAR(COMP.NRCOMPSERVICO) AS IDARVORE,
                   TO_CHAR(COMP.NRCOMPSERVICOPAI) AS IDARVOREPAI,
                   COMP.NRCOMPSERVICO,
                   COMP.NRCOMPSERVICO AS CDELEMENTO,
                   COMP.NMCOMPSERVICO AS NMELEMENTO,
                   NULL AS IDTIPOELEMENTO,
                   -1 AS IDORDENAELEMENTO,
                   NULL AS CDTPMEDIDAVARIAVEL,
                   (SELECT TO_CHAR(COMP.NRCOMPSERVICO) FROM DUAL) AS IDELEMENTO,
                   NULL AS IDELEMENTOPAI,
                   COMP.IDCOMPOBRIGATORIA AS IDSERVOBRIGCOMP,
                   COMP.NRORG,
                   'N' AS IDESCOLHIDO,
                   'C' AS TIPO,
                   NVL(QTPAI.QTSEGUNDA, NVL(QTSERV.QTSEGUNDA, 0)) AS QTSEGUNDA,
                   NVL(QTPAI.QTTERCA, NVL(QTSERV.QTTERCA, 0)) AS QTTERCA,
                   NVL(QTPAI.QTQUARTA, NVL(QTSERV.QTQUARTA, 0)) AS QTQUARTA,
                   NVL(QTPAI.QTQUINTA, NVL(QTSERV.QTQUINTA, 0)) AS QTQUINTA,
                   NVL(QTPAI.QTSEXTA, NVL(QTSERV.QTSEXTA, 0)) AS QTSEXTA,
                   NVL(QTPAI.QTDOMINGO, NVL(QTSERV.QTDOMINGO, 0)) AS QTDOMINGO,
                   NVL(QTPAI.QTSABADO, NVL(QTSERV.QTSABADO, 0)) AS QTSABADO,
                   NVL(QTPAI.QTMENSAL, NVL(QTSERV.QTMENSAL, 0)) AS QTMENSAL,
                   NVL(QTPAI.QTFIMSEMANA, (NVL(QTSERV.QTDOMINGO, 0) + NVL(QTSERV.QTSABADO, 0))) AS QTFIMSEMANA,
                   NVL(QTPAI.QTDIAUTIL, (NVL(QTSERV.QTSEGUNDA, 0) + NVL(QTSERV.QTTERCA, 0) + NVL(QTSERV.QTQUARTA, 0) + NVL(QTSERV.QTQUINTA, 0) + NVL(QTSERV.QTSEXTA, 0))) AS QTDIAUTIL,
                   NVL(QTSERV.HRENTRADA, '00:00') AS HRENTRADA,
                   NVL(QTSERV.HRSAIDA, '00:00') AS HRSAIDA
                FROM FCT_COMPSERVICO COMP
                LEFT JOIN FCT_RELACQTDSERVICO QTSERV ON COMP.NRCOMPSERVICO = QTSERV.NRCOMPSERVICO
                LEFT JOIN (SELECT NRCOMPSERVICOPAI AS NRCOMPSERVICO,
                              SUM(QTSEGUNDA) AS QTSEGUNDA,
                              SUM(QTTERCA) AS QTTERCA,
                              SUM(QTQUARTA) AS QTQUARTA,
                              SUM(QTQUINTA) AS QTQUINTA,
                              SUM(QTSEXTA) AS QTSEXTA,
                              SUM(QTDOMINGO) AS QTDOMINGO,
                              SUM(QTSABADO) AS QTSABADO,
                              SUM(QTMENSAL) AS QTMENSAL,
                              SUM(QTFIMSEMANA) AS QTFIMSEMANA,
                              SUM(QTDIAUTIL) AS QTDIAUTIL
                           FROM (SELECT CONNECT_BY_ROOT FCT_COMPSERVICO.NRCOMPSERVICO AS NRCOMPSERVICOPAI,
                                     NVL(QTSERV.QTSEGUNDA, 0) AS QTSEGUNDA,
                                     NVL(QTSERV.QTTERCA, 0) AS QTTERCA,
                                     NVL(QTSERV.QTQUARTA, 0) AS QTQUARTA,
                                     NVL(QTSERV.QTQUINTA, 0) AS QTQUINTA,
                                     NVL(QTSERV.QTSEXTA, 0) AS QTSEXTA,
                                     NVL(QTSERV.QTDOMINGO, 0) AS QTDOMINGO,
                                     NVL(QTSERV.QTSABADO, 0) AS QTSABADO,
                                     NVL(QTSERV.QTMENSAL, 0) AS QTMENSAL,
                                     NVL((QTDOMINGO + QTSABADO), 0) AS QTFIMSEMANA,
                                     NVL(QTSEGUNDA, 0) + NVL(QTTERCA, 0) + NVL(QTQUARTA, 0) + NVL(QTQUINTA, 0) + NVL(QTSEXTA, 0) AS QTDIAUTIL
                              FROM FCT_RELACQTDSERVICO QTSERV
                              INNER JOIN FCT_COMPSERVICO ON QTSERV.NRCOMPSERVICO = FCT_COMPSERVICO.NRCOMPSERVICO 
                              CONNECT BY PRIOR FCT_COMPSERVICO.NRCOMPSERVICO = FCT_COMPSERVICO.NRCOMPSERVICOPAI 
                              START WITH FCT_COMPSERVICO.NRCOMPSERVICO IN(SELECT NRCOMPSERVICO
                                                                           FROM FCT_COMPSERVICO
                                                                           WHERE NRCOMPSERVICOPAI IS NULL))
                           GROUP BY NRCOMPSERVICOPAI) QTPAI 
                ON COMP.NRCOMPSERVICO = QTPAI.NRCOMPSERVICO
                WHERE COMP.NRCOMPSERVICO IN (SELECT NRCOMPSERVICO
                                             FROM FCT_RELORCAMODCOMPOSICAO
                                             WHERE NRORCAMENTOMODELO = $nrmodelo)
                UNION ALL
                SELECT DISTINCT TO_CHAR(NRCOMPSERVICO) || '_' || IDELEMENTO AS IDARVORE,
                    DECODE(IDELEMENTOPAI, NULL, TO_CHAR(NRCOMPSERVICO), TO_CHAR(NRCOMPSERVICO) || '_' || IDELEMENTOPAI) AS IDARVOREPAI,
                    NRCOMPSERVICO,
                    CDELEMENTO,
                    NMELEMENTO,
                    IDTIPOELEMENTO,
                    IDORDENAELEMENTO,
                    CDTPMEDIDAVARIAVEL,
                    IDELEMENTO AS IDELEMENTO,
                    IDELEMENTOPAI,
                    IDSERVOBRIGCOMP,
                    NRORG,
                    IDESCOLHIDO,
                    'E' AS TIPO,
                    0 AS QTSEGUNDA,
                    0 AS QTTERCA,
                    0 AS QTQUARTA,
                    0 AS QTQUINTA,
                    0 AS QTSEXTA,
                    0 AS QTDOMINGO,
                    0 AS QTSABADO,
                    0 AS QTMENSAL,
                    0 AS QTFIMSEMANA,
                    0 AS QTDIAUTIL,
                    '00:00' AS HRENTRADA,
                    '00:00' AS HRSAIDA
                FROM (SELECT COMPOSICAO.NRCOMPSERVICO,
                          COMPOSICAO.NRITCOMPSERVICO,
                          COMPOSICAO.CDELEMENTO,
                          COMPOSICAO.NMELEMENTO,
                          COMPOSICAO.IDTIPOELEMENTO,
                          COMPOSICAO.IDORDENAELEMENTO,
                          COMPOSICAO.CDTPMEDIDAVARIAVEL,
                          COMPOSICAO.IDELEMENTO,
                          COMPOSICAO.IDELEMENTOPAI,
                          COMPOSICAO.IDSERVOBRIGCOMP,
                          COMPOSICAO.NRORG,
                          'N' AS IDESCOLHIDO
                       FROM
                         (SELECT ITEM.NRCOMPSERVICO,
                             ITEM.NRITCOMPSERVICO,
                             ITEM.CDSERVORCA AS CDELEMENTO,
                             ELEMENTO.NMSERVORCA AS NMELEMENTO,
                             ELEMENTO.IDTIPOELEMENTO,
                             ELEMENTO.IDORDENAELEMENTO,
                             ELEMENTO.CDTPMEDIDAVARIAVEL,
                             ITEM.IDELEMENTO,
                             ITEM.IDELEMENTOPAI,
                             ITESCOPO.NRTIPOORCAMENTO,
                             ITPRECO.VRPRECO,
                             ITEM.IDSERVOBRIGCOMP,
                             ITEM.NRORG
                          FROM FCT_ITCOMPSERVICO ITEM
                          INNER JOIN FCT_SERVICOORCA ELEMENTO ON ITEM.IDELEMENTO = ELEMENTO.IDELEMENTO
                          AND ITEM.NRORG = ELEMENTO.NRORG
                          LEFT JOIN FCT_RELACSERVICO SERVRELAC ON ITEM.CDSERVORCA = SERVRELAC.CDSERVORCA
                          AND ITEM.NRORG = SERVRELAC.NRORG
                          LEFT JOIN FCT_ITESCOPOPROPOSTA ITESCOPO ON SERVRELAC.CDSERVORCARELAC = ITESCOPO.CDSERVORCA
                          AND SERVRELAC.NRORG = ITESCOPO.NRORG
                          AND ITESCOPO.NRTIPOORCAMENTO IN ($nrtipoorcamento)
                          LEFT JOIN FCT_TIPOORCAMENTO ESCOPO ON ITESCOPO.NRTIPOORCAMENTO = ESCOPO.NRTIPOORCAMENTO
                          AND ITESCOPO.NRORG = ESCOPO.NRORG
                          LEFT JOIN FCT_ITPRECBASESERV ITPRECO ON ESCOPO.CDPRECBASESERV = ITPRECO.CDPRECBASESERV
                          AND ESCOPO.NRORG = ITPRECO.NRORG
                          AND ITESCOPO.CDSERVORCA = ITPRECO.CDSERVORCA
                          WHERE ITEM.NRCOMPSERVICO IN (SELECT NRCOMPSERVICO
                                                       FROM FCT_RELORCAMODCOMPOSICAO
                                                       WHERE NRORCAMENTOMODELO = $nrmodelo)
                          ORDER BY ITEM.CDSERVORCA) COMPOSICAO
                       GROUP BY COMPOSICAO.NRCOMPSERVICO,
                                COMPOSICAO.NRITCOMPSERVICO,
                                COMPOSICAO.CDELEMENTO,
                                COMPOSICAO.NMELEMENTO,
                                COMPOSICAO.IDTIPOELEMENTO,
                                COMPOSICAO.IDORDENAELEMENTO,
                                COMPOSICAO.CDTPMEDIDAVARIAVEL,
                                COMPOSICAO.IDELEMENTO,
                                COMPOSICAO.IDELEMENTOPAI,
                                COMPOSICAO.IDSERVOBRIGCOMP,
                                COMPOSICAO.NRORG
                       ORDER BY COMPOSICAO.NRITCOMPSERVICO)";
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }*/
    /**
     * Atualiza o valor total da proposta com a soma dos itens.
     **/
    public function atualizarValorProposta(array $params) {
        $sql = "UPDATE FCT_ORCAMENTO
                   SET VRORCAMENTO    = :VRORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Retorna a soma dos itens da proposta.
     */
    public function getSomaValorItensProposta(array $params) {
        $sql = "SELECT NVL(SUM(VRTOTSERV), 0) AS VRTOTALITEMORCA
                  FROM FCT_RESULTORCALIMENT
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * busca e retorna os dados do servico ligado a composição da proposta
     **/
    public function getDadosServicoRelac(array $params, $nrtipoorcamento, $nrcomposicao) {
        $sql = "SELECT DISTINCT SERVICO.NMSERVORCA, 
                    ESCOPO.NRTIPOORCAMENTO
                FROM FCT_ITCOMPSERVICO ITEM
                LEFT JOIN FCT_RELACSERVICO SERVRELAC
                    ON ITEM.CDSERVORCA = SERVRELAC.CDSERVORCA
                    AND ITEM.NRORG     = SERVRELAC.NRORG
                LEFT JOIN FCT_SERVICOORCA ELEMENTO
                    ON ITEM.CDSERVORCA      = ELEMENTO.CDSERVORCA
                    AND ITEM.NRORG          = ELEMENTO.NRORG 
                LEFT JOIN FCT_ITESCOPOPROPOSTA ITESCOPO
                    ON SERVRELAC.CDSERVORCARELAC = ITESCOPO.CDSERVORCA
                    AND SERVRELAC.NRORG          = ITESCOPO.NRORG
                LEFT JOIN FCT_TIPOORCAMENTO ESCOPO
                    ON ITESCOPO.NRTIPOORCAMENTO = ESCOPO.NRTIPOORCAMENTO
                    AND ITESCOPO.NRORG          = ESCOPO.NRORG
                LEFT JOIN FCT_SERVICOORCA SERVICO
                    ON SERVRELAC.CDSERVORCARELAC = SERVICO.CDSERVORCA
                    AND SERVRELAC.NRORG          = SERVICO.NRORG
                WHERE CDSERVORCARELAC = :CDSERVORCA
                    AND ITEM.NRORG    = :NRORG
                    AND ESCOPO.NRTIPOORCAMENTO IN($nrtipoorcamento)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $dataSet = $std->fetchAll();
        return isset($dataSet) ? $dataSet : null;
    }
    /**
     * insere os impostos ligados ao orçamento que está sendo 
     * criado
    **/
    public function inserirImpostoProposta(array $params){
        $sql = "INSERT INTO FCT_RELACIMPOSTOORCA(NRIMPOSTOORCA, NRSEQORCAMENTO, CDIMPOSTO, 
                    VRPERCIMPOSTO, NRORG,        IDATIVO,   NRORGINCLUSAO,      CDOPERINCLUSAO, 
                    NRORGULTATU,   CDOPERULTATU, DTINCLUSAO, DTULTATU)
                VALUES(:NRIMPOSTOORCA, :NRSEQORCAMENTO, :CDIMPOSTO, 
                        :VRPERCIMPOSTO, :NRORG,         :IDATIVO, :NRORGINCLUSAO, :CDOPERINCLUSAO, 
                        :NRORGULTATU,   :CDOPERULTATU,   SYSDATE,  SYSDATE)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * retona o valor total da mão de obra de um orcamento
    **/
    public function getVrMObraProposta(array $params){
        $sql = "SELECT SUM(NVL(FORMPROP.QTFORMACAOPROP, 1) * NVL(ITAUXFORM.VRTOTAL, 0)) AS VRTOTAL
                FROM FCT_CALCFORMPRECSERV_ITPRO ITAUXFORM
                INNER JOIN FCT_RELACFORMACAOPROP FORMPROP
                  ON ITAUXFORM.NRFORMPRECOSERVICO   = FORMPROP.NRFORMPRECOSERVICO
                  AND ITAUXFORM.NRORG               = FORMPROP.NRORG
                  AND ITAUXFORM.NRRELACFORMACAOPROP = FORMPROP.NRRELACFORMACAOPROP
                WHERE FORMPROP.NRSEQORCAMENTO     = :NRSEQORCAMENTO
                    AND FORMPROP.NRORG              = :NRORG
                    AND NRCALCFORMPRECSERVPAI      IS NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $dataSet = $std->fetchAll();
        return isset($dataSet[0]) ? $dataSet[0]["VRTOTAL"] : 0;
    }
    /**
     * Retorna o valor total do investimento de uma proposta
    **/
    public function getVrInvestProposta(array $params){
        $sql = "SELECT SUM(NVL(IVPROP.VRPARCELA, 0)) AS VRPARCELA
                FROM FCT_RELACINVESTPROPOSTA IVPROP
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $dataSet = $std->fetchAll();
        return isset($dataSet[0]) ? $dataSet[0]["VRPARCELA"] : 0;
    }
    /**
     * Retorna o valor total das despesas de uma proposta
    **/
    public function getVrDespesasProposta(array $params){
        $sql = "SELECT SUM(NVL(DESP.VRDESPESAORCA, 0)) AS VRDESPESAORCA
                FROM FCT_RELACDESPPROPOSTA DESP
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $dataSet = $std->fetchAll();
        return isset($dataSet[0]) ? $dataSet[0]["VRDESPESAORCA"] : 0;
    }
    /**
     * Retorna o valor total das taxas de uma proposta
    **/
    public function getVrTaxasProposta(array $params){
        $sql = "SELECT SUM(NVL(TAXA.VRPERCTAXAORC, 0)) AS VRPERCTAXAORC
                FROM FCT_RELACTAXAPROPOSTA TAXA
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $dataSet = $std->fetchAll();
        return isset($dataSet[0]) ? $dataSet[0]["VRPERCTAXAORC"] : 0;
    }
    /**
     * Retorna o valor total dos impostos de uma proposta
    **/
    public function getVrImpostosProposta(array $params){
        $sql = "SELECT SUM(NVL(IMPOSTO.VRPERCIMPOSTO, 0)) AS VRPERCIMPOSTO
                FROM FCT_RELACIMPOSTOORCA IMPOSTO
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $dataSet = $std->fetchAll();
        return isset($dataSet[0]) ? $dataSet[0]["VRPERCIMPOSTO"] : 0;
    }
    /**
     * Retorna o valor total dos serviços de uma proposta
    **/
    public function getVrServicosProposta(array $params){
        $sql = "SELECT SUM(NVL(SERV.VRTOTALITEMORCA, 0)) AS VRTOTALITEMORCA
                FROM FCT_ITEMORCAMENTO SERV
                JOIN FCT_ORCAMENTO ORC 
                    ON SERV.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                    AND SERV.NRORG         = ORC.NRORG
                WHERE SERV.IDELEMENTO IS NOT NULL
                  AND SERV.NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND SERV.NRORG          = :NRORG
                GROUP BY ORC.QTDIASVALIDADE";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $dataSet = $std->fetchAll();
        return isset($dataSet[0]) ? $dataSet[0]["VRTOTALITEMORCA"] : 0;
    }
    /**
     * Retorna o valor total dos serviços de uma proposta
    **/
    public function getVrMarkup(array $params){
        $sql = "SELECT NVL(ORC.VRPERCMARKUP, 0) AS VRPERCMARKUP,
                    NVL(ORC.VRPERCMARKUPAPLICADO, 0) AS VRPERCMARKUPAPLICADO
                FROM FCT_ORCAMENTO ORC
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $dataSet = $std->fetchAll();
        return isset($dataSet[0]) ? $dataSet[0] : 0;
    }
    /**
     * Calcula o valor total de cada serviço do orcamento
    **/
    public function getInfoServicosProp(array $params){
        $sql = "SELECT SERVPROP.QTTOTALSERV,
                  SERVPROP.VRCUSTOINFORMADO,
                  SERVPROP.VRCUSTOIDEAL,
                  SERVPROP.VRTOTSERV,
                  SUM(SERVPROP.VRCUSTOSERV) AS VRCUSTOSERV,
                  SERVPROP.NRCOMPSERVICO,
                  SERVPROP.IDELEMENTORAIZ,
                  SUM(SERVPROP.QTTOTALORCA) AS QTTOTALORCA,
                  SERVORCA.NMSERVORCA AS NMELEMENTO,
                  SERVPROP.VRPERCSERVICO,
                  LISTAGG(SERVPROP.NRTIPOORCAMENTO, '_') WITHIN GROUP (ORDER BY SERVPROP.NRTIPOORCAMENTO) AS NRTIPOORCAMENTO,
                  LISTAGG(SERVPROP.CDSERVORCA, '_') WITHIN GROUP (ORDER BY SERVPROP.CDSERVORCA) AS CDSERVORCA
                FROM (SELECT RT.QTTOTALSERV,
                        RT.VRTOTSERV,
                        SERV.VRCUSTOSERV, 
                        RT.VRCUSTOIDEAL,
                        RT.VRCUSTOINFORMADO,
                        SERV.NRORG,
                        SERV.NRCOMPSERVICO,
                        SERV.CDSERVORCA,
                        SERV.NRTIPOORCAMENTO,
                        RT.VRPERCSERVICO,
                        RT.IDELEMENTORAIZ,
                        (SELECT NVL(SUM(QTMENSAL), 0)
                          FROM FCT_RESULTORCALIMENT
                          WHERE NRSEQORCAMENTO = RT.NRSEQORCAMENTO) AS QTTOTALORCA
                      FROM FCT_RELACSERVORCAMENTO SERV
                      INNER JOIN FCT_RESULTORCALIMENT RT
                        ON SERV.NRSEQORCAMENTO = RT.NRSEQORCAMENTO
                        AND SERV.NRORG         = RT.NRORG
                        AND SERV.NRCOMPSERVICO = RT.NRCOMPSERVICORAIZ
                      WHERE SERV.NRSEQORCAMENTO = :NRSEQORCAMENTO
                        AND SERV.NRORG          = :NRORG) SERVPROP
                INNER JOIN FCT_SERVICOORCA SERVORCA
                  ON SERVPROP.IDELEMENTORAIZ = SERVORCA.IDELEMENTO
                  AND SERVPROP.NRORG = SERVORCA.NRORG
                GROUP BY SERVPROP.NRCOMPSERVICO,
                  SERVPROP.IDELEMENTORAIZ,
                  SERVPROP.QTTOTALSERV,
                  SERVORCA.NMSERVORCA,
                  VRCUSTOINFORMADO,
                  VRCUSTOIDEAL,
                  VRTOTSERV,
                  VRPERCSERVICO";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Retorna os itens da proposta.
     */
    public function getItensProposta(array $params) {
        $sql = "SELECT ITEMPAI. NRITEMORCAMENTO,
                  ITEM.CDSERVORCA,
                  ITEM.NRTIPOORCAMENTO,
                  (SELECT SUM(VRTOTALITEMORCA) 
                    FROM FCT_ITEMORCAMENTO 
                    WHERE NRSEQORCAMENTO = ITEM.NRSEQORCAMENTO
                      AND IDELEMENTO IS NULL
                      AND NRORG = ITEM.NRORG) AS VRORCAMENTO,
                  ITEMPAI.VRTOTALITEMORCA,
                  (ITEM.VRUNITITEMORCA * 100/(SELECT SUM(VRUNITITEMORCA) 
                                              FROM FCT_ITEMORCAMENTO 
                                              WHERE NRSEQORCAMENTO = ITEM.NRSEQORCAMENTO
                                                AND IDELEMENTO     = ITEM.IDELEMENTO
                                                AND NRORG          = ITEM.NRORG)) VRPERCITEMORCA
                FROM FCT_ITEMORCAMENTO ITEM
                INNER JOIN FCT_ORCAMENTO ORC
                  ON ITEM.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                  AND ITEM.NRORG         = ORC.NRORG
                INNER JOIN FCT_ITEMORCAMENTO ITEMPAI
                  ON ITEM.NRSEQORCAMENTO = ITEMPAI.NRSEQORCAMENTO
                  AND ITEM.NRORG         = ITEMPAI.NRORG
                  AND ITEM.NRITEMORCAMENTOPAI = ITEMPAI.NRITEMORCAMENTO
                INNER JOIN FCT_RESULTORCALIMENT RESUL
                  ON ITEM.NRSEQORCAMENTO = RESUL.NRSEQORCAMENTO
                  AND ITEM.NRORG         = RESUL.NRORG
                  AND ITEM.IDELEMENTO    = RESUL.IDELEMENTORAIZ
                WHERE ORC.NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND ORC.NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Retorna os itens da proposta.
     */
    public function alterarResultadoProposta(array $params) {
        $sql = "UPDATE FCT_RESULTORCALIMENT
                SET VRTOTSERV = (:VRTOTAL * VRPERCSERVICO/100),
                    VRCUSTOINFORMADO = (:VRCUSTOINFO * (VRPERCSERVICO/100)),
                    VRCUSTOIDEAL     = (:VRCUSTOIDEAL * (VRPERCSERVICO/100)),
                    QTTOTALSERV      = :QTTOTAL,
                    CDOPERULTATU     = :CDOPERULTATU
                WHERE NRSEQORCAMENTO    = :NRSEQORCAMENTO
                  AND NRCOMPSERVICORAIZ = :NRCOMPSERVICORAIZ
                  AND NRORG             = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Retorna a quantidade e o valor total dos elementos 
     * de uma proposta
    **/
    public function getInfoElementosServicosProp(array $params){
        $sql = "SELECT NVL(SUM(QTMENSAL), 0) AS QTMENSAL, NRCOMPSERVICO
                FROM FCT_RELACSERVORCAMENTO
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG          = :NRORG
                  AND IDATIVO        = 'S'
                  AND IDESCOLHIDO    = 'S'
                  AND IDELEMENTOPAI IS NULL
                GROUP BY NRCOMPSERVICO";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $dataSet = $std->fetchAll();
        return isset($dataSet) ? $dataSet : ["NRCOMPSERVICO" => null, "QTTOTAL" => 1];
    }
    /**
     * atualiza os itens da proposta.
     */
    public function atualizarValorItem(array $params) {
        $sql = "UPDATE FCT_ITEMORCAMENTO
                SET VRUNITITEMORCA  = VRTOTALITEMORCA + :ACRESCIMO,
                    VRTOTALITEMORCA = VRTOTALITEMORCA + :ACRESCIMO,
                    VRITEMSEMDESC   = VRTOTALITEMORCA + :ACRESCIMO,
                    CDOPERULTATU    = :CDOPERULTATU
                WHERE NRSEQORCAMENTO    = :NRSEQORCAMENTO
                  AND NRITEMORCAMENTO   = :NRITEMORCAMENTO
                  AND NRORG             = :NRORG
                  AND IDELEMENTO IS NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Atualiza a situação das propostas informadas.
     * 
     * @param string  $situacao
     * @param string  $nrseqorcamento
     * @param integer $nrorg
     */
    public function atualizarSituacaoPropostas($situacao, $nrseqorcamento, $nrorg) {
        $sql = "UPDATE FCT_ORCAMENTO
                   SET IDSITUACAOORCAMENTO = :IDSITUACAOORCAMENTO
                 WHERE NRSEQORCAMENTO IN (".$nrseqorcamento.")
                   AND NRORG          = :NRORG";
        $params = array(
            "IDSITUACAOORCAMENTO" => $situacao,
            "NRORG"               => $nrorg
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Atualiza a Proposta ao aceitá-la.
     * 
     * @param mixed[] $params
     */
    public function atualizarPropostaAceite(array $params) {
        $sql = "UPDATE FCT_ORCAMENTO
                   SET IDSITUACAOORCAMENTO = :IDSITUACAOORCAMENTO,
                       DTACEITEORCA        = :DTACEITEORCA,
                       CDOPERULTATU        = :CDOPERULTATU,
                       DTULTATU            = SYSDATE
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Retorna o código da ultima pk inserida na tabela de fct_relacformacaoprop 
    **/
    public function getCodeRelacFormacaoProp(){
        $sql = "SELECT NVL(MAX(NRRELACFORMACAOPROP), 1) NRRELACFORMACAOPROP
                FROM FCT_RELACFORMACAOPROP";
        $std = $this->connection->prepare($sql);
        $std->execute();
        $dataSet = $std->fetchAll();
        return isset($dataSet) ? $dataSet[0]["NRRELACFORMACAOPROP"] : 0;
    }
    /**
     * Altera os valores na tabela de resultado de acordo com 
     * a alteração do valor do servico de uma proposta ja inserida
    **/
    public function updateResultadoProposta(array $params){
        $sql = "UPDATE FCT_RESULTORCALIMENT
                SET VRCUSTOINFORMADO = :VRSERVICO,
                    CDOPERULTATU     = :CDOPERADOR,
                    NRORGULTATU      = :NRORG,
                    VRPERCSERVICO    = :VRPERCSERVICO,
                    VRTOTSERV        = QTTOTALSERV * :VRSERVICO
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND NRORG        = :NRORG
                    AND NRCOMPSERVICORAIZ = :NRCOMPOSICAO";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Atualiza o valor do markup aplicado na proposta
     **/
    public function atualizarMarkupAplicado(array $params) {
        $sql = "UPDATE FCT_ORCAMENTO
                   SET VRPERCMARKUPAPLICADO = :VRPERCMARKUPAPLICADO
                 WHERE NRSEQORCAMENTO       = :NRSEQORCAMENTO
                   AND NRORG                = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Monta e retorna um array de elementos que serão inseridos na 
     * tabela de itens 
    **/
    public function getItensTipoElementoProposta(array $params){
        $sql = "SELECT NVL(VRCUSTOSERV, 0) AS VRCUSTOSERV,
                  ELEMENTOS.IDELEMENTOPAI,
                  ELEMENTOS.CDSERVORCA,
                  ELEMENTOS.NRTIPOORCAMENTO,
                  ELEMENTOS.CDTPMEDIDASERV,
                  ELEMENTOS.QTMENSAL,
                  SERV.NMSERVORCA AS NMELEMENTOPAI
                FROM (SELECT CDSERVORCA,
                        NVL(VRTOTMENSALSERV, 0) AS VRTOTMENSALSERV,
                        CONNECT_BY_ROOT IDELEMENTO AS IDELEMENTOPAI,
                        NRTIPOORCAMENTO, NRORG,
                        VRCUSTOSERV,
                        QTMENSAL,
                        CDTPMEDIDASERV
                      FROM FCT_RELACSERVORCAMENTO
                      WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                        AND NRORG          = :NRORG
                      CONNECT BY PRIOR IDELEMENTO = IDELEMENTOPAI
                      START WITH IDELEMENTO IN(SELECT IDELEMENTO 
                                                FROM FCT_RELACSERVORCAMENTO
                                                WHERE IDELEMENTOPAI IS NULL
                                                  AND NRSEQORCAMENTO = :NRSEQORCAMENTO
                                                  AND NRORG          = :NRORG)) ELEMENTOS
                INNER JOIN FCT_SERVICOORCA SERV
                  ON ELEMENTOS.CDSERVORCA = SERV.CDSERVORCA
                  AND ELEMENTOS.NRORG     = SERV.NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
}